/*
    cve-manager : CVE management tool
    Copyright (C) 2017-2024 Alexey Appolonov

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef     lcve_manager_confH
#define     lcve_manager_confH

#include    <algorithm>

#include    "manager.h"

#define     DB_CON_SEC     "db_connection"
#define     COMMON_SEC     "common"

const set<string> common_params_file_paths = {"basealt", "download"};

//-----------------------------------------------------------------------------
// Main configuration

class TCVEMConf : public TConf{

    protected:

        map<string, string> db_connection = {
            {"host", ""},
            {"user",   ""},
            {"password", ""},
            {"database", ""},
            };
        map<string, string> common = {
            {"basealt",       ""},
            {"download",      ""},
            {"master_branch", ""},
            };
        vector<string> excluded_vulsrc;
        map<string, map<string, string>> branches;

        // Apply configuration that was read from the conf file
        bool ParseConfDOM(string &err);
        // Parse the database section
        bool ParseDBSec(string &err);
        // Parse the common section
        bool ParseCommonSec(string &err);

    public:

        // Element access
        const map<string, string>& DBConParams(void) const{
            return db_connection;
            }
        const map<string, string>& CommonParams(void) const{
            return common;
            }
        const vector<string>& ExclVulSrc(void) const{
            return excluded_vulsrc;
            }
        const map<string, map<string, string>>& Branches(void) const{
            return branches;
            }
        // Configure the module
        bool Configure(string &err);
    };

//-----------------------------------------------------------------------------

#endif
