/*
    cve-manager : CVE management tool
    Copyright (C) 2017-2024 Alexey Appolonov

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef     lcve_manager_databaseH
#define     lcve_manager_databaseH

// MySQL Connector/C++
#include    <mysql_connection.h>
#include    <cppconn/driver.h>
#include    <cppconn/exception.h>
#include    <cppconn/resultset.h>
#include    <cppconn/statement.h>

#include    "manager.h"

//-----------------------------------------------------------------------------
// MySQL DB interface

class TDatabase{

    protected:

        string host;
        string user;
        string password;
        string database;

        sql::Driver *driver  = NULL; // Pointer to the object used to make connection with DB
        sql::Connection *con = NULL; // P... used to serve connection with DB (created by the driver)
        sql::Statement *stmt = NULL; // P... used to send SQL queries
        sql::ResultSet *res  = NULL; // P... used to process query respose

        // Create connection
        bool Connect(string &err, const bool force=false);
        // Send given query to the DB and return the result
        sql::ResultSet* GetResult(const string &query, string &err);
        // Fetch all results into matrix of strings
        vector<vector<string>> FetchAll(sql::ResultSet *res, const size_t n_cols);
        // Check DB or table existence
        bool ExistenceCheck(const string &query, string &err);
        // Check existence of the DB
        bool DBExists(string &err);
        // Get names of all tables of the DB
        bool GetTableNames(vector<string> &table_names, string &err);
        // Check that table is not empty
        bool TableNotEmpty(const string &name, string &err);
        // Create the DB
        bool CreateDB(string &err);
        // Use the DB
        bool UseDB(string &err);
        // Delete DB and create empty DB with the same name
        bool ClearDB(void);
        // Copy structure of the table but not any of it's rows
        bool CopyTable(const string &table, const string &new_table, string &err);
        // Delete table with a given name
        bool DropTable(const string &table, string &err);
        // Drop tables that have specified names
        bool DropChildTables(const vector<string> &tables, string &err);
        // Push table name into vector of table names if there is a table with this
        // name in the DB
        bool PushTable(vector<string> &tables, const string &table, string &err);
        // Format given SQL exception message
        string FormatSQLException(const sql::SQLException &e) const;
        // Memory deallocation
        void Clear(void);

    public:

        // Get name of the DB type
        string Name(void){
            return "MySQL DB";
            }
        // Setup the DB connection params
        bool Configure(const map<string, string> &db_connection, string &err);
        // Check table existence in the DB
        bool TableExists(const string &name, string &err);
        // Send noreturn query to the DB
        bool Execute(const string &query, string &err);

        ~TDatabase(void){
            Clear();
            }
    };

//-----------------------------------------------------------------------------

#endif
