/*
    cve-manager : CVE management tool
    Copyright (C) 2017-2024 Alexey Appolonov

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef     lcve_manager_vulH
#define     lcve_manager_vulH

#include    <set>
#include    <regex>

#include    "manager.h"

using namespace std;

//-----------------------------------------------------------------------------

const int cur_year = CurrentYear();
const int cur_dc   = (cur_year - 2000) / 10;
const int prev_dc  = cur_dc - 1;
const int year_of_a_cur_dc = cur_year - 2000 - cur_dc * 10;

//-----------------------------------------------------------------------------

const regex fixes_regex(
    "(.*)"                  // match(es) 1
	"(fix(e[sd])?(:|for)?)" // match(es) 2-4
	"(.*)?"                 // match(es) 5
    );

const regex negation_regex("(\\snot?|n\\'t)\\s+$");

const string year_regex_str = [](void){
    ostringstream oss;
    oss << "(20(" << cur_dc
            << "[0-" << year_of_a_cur_dc << "]|"
            << "[0-" << prev_dc << "][0-9]"
        << ")|199[1-9])";
    return oss.str();
    }();

const regex vul_id_regex("\\s*"
	"((cve\\-|bdu:)" + year_regex_str + "\\-[0-9]{4,7})"
	"(\\s*((,\\s?)?and|,|\\s)\\s*)?"
	);

//-----------------------------------------------------------------------------

// Get IDs of vulnerabilities from a given multiline text
pair<map<string, set<string>>, string> GetFixedVerAndVulIDs(const string &txt);

//-----------------------------------------------------------------------------

#endif
